define( [ 'require', 'lodash'
    , '$App/mapping/transformation'
    ], function( require, _ ){
    var Transformation = require( '$App/mapping/transformation' );
	return {
        get: function (path, data) {
            if (path === undefined) throw new Error("Path is undefined");

            if (path === "$") return data;

            return this._get(this.validateAndPreparePath(path), data);
        },
        _get: function (pathArray, data) {
            if (pathArray.length === 0) return data;
            if (data === undefined) return undefined;

            var accessor,
                pathHead = _.head(pathArray),
                pathTail = _.tail(pathArray),
                nodeType = this.getPathNodeType(pathHead),
                arrayLevel = (nodeType === this.NODE_TYPES.INDEX || nodeType === this.NODE_TYPES.ALL),
                objectLevel = (nodeType === this.NODE_TYPES.PROPERTY),
                pathDynamic = (nodeType === this.NODE_TYPES.ALL),
                pathStatic = (nodeType === this.NODE_TYPES.INDEX || nodeType === this.NODE_TYPES.PROPERTY);

            if (_.isArray(data) && objectLevel) throw new Error("Attempt to access Array by property name : " + pathHead);
            if (_.isTrueObject(data) && arrayLevel) throw new Error("Attempt to access Object by index: " + pathHead);

            if (pathDynamic) {
                for (var j = 0; j < data.length; j++) {
                    if (pathArray[1] !== undefined) {
                        pathArray[0] = j.toString();
                        data[j] = this._get(pathArray, data);
                    }
                }
                return data;
            } else if (pathStatic) {
                if (nodeType === this.NODE_TYPES.INDEX) {
                    accessor = parseInt(pathHead);
                } else if (nodeType === this.NODE_TYPES.PROPERTY) {
                    accessor = this.getPropName(pathHead);
                }

                return this._get(pathTail, data ? data[accessor] : data);
            } else {
                throw new Error("Unsupported node type " + nodeType);
            }
        },

        merge: function (path, originalData, additionalData, transformation) {
            if (_.isUndefined(transformation)) {
                transformation = new Transformation();
            }

            if (path === "$") {
                var buf;

                if (_.isArray(additionalData)) {
                    buf = [];
                    _.each(additionalData, function (elt, i) {
                        buf[i] = transformation.transform(elt);
                    });
                } else {
                    buf = transformation.transform(additionalData);
                }

                return buf;
            }

            return this._merge(this.validateAndPreparePath(path), originalData, additionalData, transformation);
        },
        _merge: function (pathArray, baseValue, addValue, transformation) {
            var buf;
            if (pathArray.length === 0) {
                if (_.isArray(addValue)) {
                    buf = [];
                    _.each(addValue, function (elt, i) {
                        buf[i] = transformation.transform(elt);
                    });
                } else {
                    buf = transformation.transform(addValue);
                }

                return buf;
            }

            var accessor,
                pathHead = _.head(pathArray),
                pathTail = _.tail(pathArray),
                nodeType = this.getPathNodeType(pathHead),
                arrayLevel = (nodeType === this.NODE_TYPES.INDEX || nodeType === this.NODE_TYPES.ALL),
                objectLevel = (nodeType === this.NODE_TYPES.PROPERTY),
                pathDynamic = (nodeType === this.NODE_TYPES.ALL),
                pathStatic = (nodeType === this.NODE_TYPES.INDEX || nodeType === this.NODE_TYPES.PROPERTY);


            // Initiating result
            var result;
            if (arrayLevel) {
                result = _.isArray(baseValue) ? baseValue : [];
            } else if (objectLevel) {
                result = Object.prototype.toString.call(baseValue) === '[object Object]' ? baseValue : {};
            } else {
                throw new Error("Unsupported node type " + nodeType);
            }

            // Extending result
            if (pathDynamic) {
                if (!_.isArray(addValue)) {
                    throw new Error(JSON.stringify(addValue) + " expected to be an Array");
                }

                var dynamicResult = [];

                for (var i = 0; i < addValue.length; i++) {
                    buf = this._merge(pathTail, result[i], addValue[i], transformation);

                    if (_isNodeMustBeMerged(pathTail, result[i], buf)) {
                        dynamicResult[i] = _.extend(result[i], buf);
                    } else {
                        dynamicResult[i] = buf;
                    }
                }

                return dynamicResult;
            } else if (pathStatic) {
                if (nodeType === this.NODE_TYPES.INDEX) {
                    accessor = parseInt(pathHead);
                } else if (nodeType === this.NODE_TYPES.PROPERTY) {
                    accessor = this.getPropName(pathHead);
                }

                buf = this._merge(pathTail, result[accessor], addValue, transformation);

                if (_isNodeMustBeMerged(pathTail, result[accessor], buf)) {
                    _.extend(result[accessor], buf);
                } else {
                    result[accessor] = buf;
                }

                return result;
            } else {
                throw new Error("Unsupported node type " + nodeType);
            }

            function _isNodeMustBeMerged(path, left, right) {
                var intermediateNode = path.length !== 0, // not a "leaf" node
                    bothObjects = (_.isTrueObject(left) && _.isTrueObject(right)),
                    bothArrays = (_.isArray(left) && _.isArray(right));

                return intermediateNode && (bothObjects || bothArrays);
            }
        },
        getPathNodeType: function (token) {
            if (token === "i") {
                return this.NODE_TYPES.ALL;
            } else if (token.indexOf('\'') <= -1) {
                return this.NODE_TYPES.INDEX;
            } else {
                return this.NODE_TYPES.PROPERTY;
            }
        },
        getPropName: function (token) {
            return token.replace(/'/g, '');
        },
        validateAndPreparePath: function (path) {
            if (!this.validatePath(path)) throw new Error('Mapping Expression is incorrect');

            var path_array = (path.substring(1, path.length)).split('][');
            path_array[0] = path_array[0].replace('[', '');
            path_array[path_array.length - 1] = path_array[path_array.length - 1].replace(']', '');
            return path_array;
        },
        validatePath: function (path) {
            return (/^\$(\[(i|\d+|'.+')\])*$/).test(path);
        },
        NODE_TYPES: {
            ALL: "all",
            INDEX: "index",
            PROPERTY: "prop"
        }
    };
});
